// ROOT
#include "TChain.h"
#include "TCanvas.h"
#include "TLegend.h"
#include "TH1F.h"
#include "TStyle.h"
// Other
#include "trktree.h"
#include "tqdm.h"

using namespace tas;

void draw(TH1F* r_peak_hist, TH1F* r_tail_hist, TH1F* r_bulk_hist, TString name)
{
    TCanvas* c = new TCanvas("c", "c", 1000, 1000);
    c->cd();
    // Normalize histograms
    r_bulk_hist->Scale(1/r_bulk_hist->Integral());
    r_peak_hist->Scale(1/r_peak_hist->Integral());
    r_tail_hist->Scale(1/r_tail_hist->Integral());
    // Draw histograms
    r_bulk_hist->Draw("hist");
    r_peak_hist->Draw("histsame");
    r_tail_hist->Draw("histsame");
    // Set line colors
    r_bulk_hist->SetLineColor(3);
    r_peak_hist->SetLineColor(1);
    r_tail_hist->SetLineColor(2);
    // Set axis labels
    r_bulk_hist->GetXaxis()->SetTitle("p_{T} [GeV]");
    r_bulk_hist->GetYaxis()->SetTitle("a.u.");
    r_bulk_hist->SetTitle("|#eta| < 2.4 and |z_{vertex}| < 20 cm");
    // Set log y
    gPad->SetLogy();
    // Create legend
    TLegend* legend = new TLegend(0.48, 0.7, 0.9, 0.9);
    legend->AddEntry(r_bulk_hist, "r_{vertex} < 18");
    legend->AddEntry(r_peak_hist, "18 #leq r_{vertex} < 20 cm");
    legend->AddEntry(r_tail_hist, "r_{vertex} #geq 20");
    legend->Draw();
    // Other
    gStyle->SetOptStat(0);
    // Save plot
    TString plotsdir = "/home/users/jguiang/public_html/dump/lst_debug_plots/";
    c->SaveAs(plotsdir+name+".png");
    c->SaveAs(plotsdir+name+".pdf");
    // Clean up
    delete c;
}

int main()
{
    // Open file
    TFile* tfile = TFile::Open(
        "/ceph/cms/store/user/evourlio/LST/samples/CMSSW_12_2_0_pre2/RelValTTbar_14TeV_CMSSW_12_5_0_pre3/event_2000.root"
    );

    // Get TTree
    TTree* ttree = (TTree*)tfile->Get("trackingNtuple/tree");

    // Init tracking ntuple reader
    trk.Init(ttree);

    // Init histograms
    TH1F* r_bulk_hist = new TH1F("r_bulk_hist", "", 100, 0, 5);
    TH1F* r_peak_hist = new TH1F("r_peak_hist", "", 100, 0, 5);
    TH1F* r_tail_hist = new TH1F("r_tail_hist", "", 100, 0, 5);

    TH1F* r_bulk_hist_elec = new TH1F("r_bulk_hist_elec", "", 100, 0, 5);
    TH1F* r_peak_hist_elec = new TH1F("r_peak_hist_elec", "", 100, 0, 5);
    TH1F* r_tail_hist_elec = new TH1F("r_tail_hist_elec", "", 100, 0, 5);

    TH1F* r_bulk_hist_muon = new TH1F("r_bulk_hist_muon", "", 100, 0, 5);
    TH1F* r_peak_hist_muon = new TH1F("r_peak_hist_muon", "", 100, 0, 5);
    TH1F* r_tail_hist_muon = new TH1F("r_tail_hist_muon", "", 100, 0, 5);

    TH1F* r_bulk_hist_pion = new TH1F("r_bulk_hist_pion", "", 100, 0, 5);
    TH1F* r_peak_hist_pion = new TH1F("r_peak_hist_pion", "", 100, 0, 5);
    TH1F* r_tail_hist_pion = new TH1F("r_tail_hist_pion", "", 100, 0, 5);

    TH1F* r_bulk_hist_kaon = new TH1F("r_bulk_hist_kaon", "", 100, 0, 5);
    TH1F* r_peak_hist_kaon = new TH1F("r_peak_hist_kaon", "", 100, 0, 5);
    TH1F* r_tail_hist_kaon = new TH1F("r_tail_hist_kaon", "", 100, 0, 5);

    // Start event loop
    int n_events_processed = 0;
    int n_events_total = ttree->GetEntries();
    tqdm bar;
    for (int entry = 0; entry < n_events_total; ++entry) 
    {
        // Runs once per event
        trk.GetEntry(entry);

        /* ---------- Event-level logic ---------- */
        for (unsigned int sim_i = 0; sim_i < sim_pt().size(); ++sim_i)
        {
            int vtx_i = trk.sim_parentVtxIdx().at(sim_i);
            
            // Check eta
            float eta = sim_eta().at(sim_i);
            if (fabs(eta) >= 2.4) { continue; }
            // Check z
            float z = simvtx_z().at(vtx_i);
            if (fabs(z) >= 20) { continue; }

            // Calculate r_vertex^2
            float x = simvtx_x().at(vtx_i);
            float y = simvtx_y().at(vtx_i);
            float r_sq = x*x + y*y + z*z;

            // Get PDG ID of simulated track
            int pdg = trk.sim_pdgId().at(sim_i);

            // Fill histograms
            float pt = sim_pt().at(sim_i);
            if (r_sq < 324) // r^2 < 18^2
            {
                r_bulk_hist->Fill(pt);
                switch(abs(pdg))
                {
                case(11):
                    r_bulk_hist_elec->Fill(pt);
                    break;
                case(13):
                    r_bulk_hist_muon->Fill(pt);
                    break;
                case(211):
                    r_bulk_hist_pion->Fill(pt);
                    break;
                case(321):
                    r_bulk_hist_kaon->Fill(pt);
                    break;
                }
            }
            else if (r_sq >= 324 && r_sq < 400) // 18^2 <= r^2 < 20^2
            {
                r_peak_hist->Fill(pt);
                switch(abs(pdg))
                {
                case(11):
                    r_peak_hist_elec->Fill(pt);
                    break;
                case(13):
                    r_peak_hist_muon->Fill(pt);
                    break;
                case(211):
                    r_peak_hist_pion->Fill(pt);
                    break;
                case(321):
                    r_peak_hist_kaon->Fill(pt);
                    break;
                }
            }
            else if (r_sq >= 400) // r^2 >= 20^2
            {
                r_tail_hist->Fill(pt);
                switch(abs(pdg))
                {
                case(11):
                    r_tail_hist_elec->Fill(pt);
                    break;
                case(13):
                    r_tail_hist_muon->Fill(pt);
                    break;
                case(211):
                    r_tail_hist_pion->Fill(pt);
                    break;
                case(321):
                    r_tail_hist_kaon->Fill(pt);
                    break;
                }
            }
        }
        /* --------------------------------------- */

        // Track progress
        bar.progress(n_events_processed, n_events_total);
        n_events_processed++;
    } 

    draw(
        r_peak_hist,
        r_tail_hist,
        r_bulk_hist,
        "pt_compare"
    );

    draw(
        r_peak_hist_elec,
        r_tail_hist_elec,
        r_bulk_hist_elec,
        "pt_compare_elec"
    );

    draw(
        r_peak_hist_muon,
        r_tail_hist_muon,
        r_bulk_hist_muon,
        "pt_compare_muon"
    );

    draw(
        r_peak_hist_pion,
        r_tail_hist_pion,
        r_bulk_hist_pion,
        "pt_compare_pion"
    );

    draw(
        r_peak_hist_kaon,
        r_tail_hist_kaon,
        r_bulk_hist_kaon,
        "pt_compare_kaon"
    );

    return 0;
}
